From 65e9389238e042d1d0de8073f4c10230eba83d39 Mon Sep 17 00:00:00 2001
From: Lucas Fialho Zawacki <lfzawacki@gmail.com>
Date: Mon, 18 Jun 2012 14:48:50 -0300
Subject: dinput: Disable linuxinput joystick based on registry key

---
 dlls/dinput/joystick_linuxinput.c |   38 +++++++++++++++++++++++++++++++++++++
 1 file changed, 38 insertions(+)

diff --git a/dlls/dinput/joystick_linuxinput.c b/dlls/dinput/joystick_linuxinput.c
index 3cab015..beca1c4 100644
--- a/dlls/dinput/joystick_linuxinput.c
+++ b/dlls/dinput/joystick_linuxinput.c
@@ -170,6 +170,42 @@ static const GUID DInput_Wine_Joystick_Base_GUID = { /* 9e573eda-7734-11d2-8d4a-
 static int have_joydevs = -1;
 static struct JoyDev *joydevs = NULL;
 
+static BOOL device_disabled_registry(const char* name)
+{
+    static const char disable_str[] = "Disable";
+    static const char all_drivers[] = "all";
+    static const char this_driver[] = "linuxinput";
+
+    char buffer[MAX_PATH+16];
+    HKEY hkey, appkey;
+    BOOL do_disable = FALSE;
+    char *disable_name;
+
+    int str_size = strlen(name) + strlen(disable_str) + 2;
+    disable_name = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, str_size);
+
+    if (!disable_name) return FALSE;
+
+    snprintf(disable_name, str_size, "%s %s", disable_str, name);
+
+    get_app_key(&hkey, &appkey);
+
+    /* Look for the "Disable controllername"="drivername" key */
+    if (!get_config_key(hkey, appkey, disable_name, buffer, sizeof(buffer)))
+    {
+        if (!strncmp(buffer, all_drivers, strlen(all_drivers)) ||
+            !strncmp(buffer, this_driver, strlen(this_driver)))
+        {
+            TRACE("Disabling joystick %s\n", name);
+            do_disable = TRUE;
+        }
+    }
+
+    HeapFree(GetProcessHeap(), 0, disable_name);
+
+    return do_disable;
+}
+
 static void find_joydevs(void)
 {
     int i;
@@ -246,6 +282,8 @@ static void find_joydevs(void)
         else
             joydev.name = joydev.device;
 
+        if (device_disabled_registry(joydev.name)) continue;
+
 	joydev.guid = DInput_Wine_Joystick_Base_GUID;
 	joydev.guid.Data3 += have_joydevs;
 
-- 
1.7.9.5

